const state = {
    loadingBankPaymentTypes: false,
    bankPaymentTypes: [],
    selectedBankPaymentType: null
}

const getters = {
    loadingBankPaymentTypes(state) {
        return state.loadingBankPaymentTypes
    },
    bankPaymentTypes(state, getters) {
        if (getters.selectedJournal && getters.selectedJournal.subtype) {
            return state.bankPaymentTypes.filter(b => b.subtype === getters.selectedJournal.subtype)
        }

        return state.bankPaymentTypes
    },
    selectedBankPaymentType (state) {
        return state.selectedBankPaymentType
    }
}

const mutations = {
    setLoadingBankPaymentTypes(state, loading) {
        state.loadingBankPaymentTypes = !!loading
    },
    setBankPaymentTypes (state, bankPaymentTypes) {
        state.bankPaymentTypes = bankPaymentTypes
    },
    setSelectedBankPaymentType (state, id) {
        if (id) {
            state.selectedBankPaymentType = this.getters.bankPaymentTypes.find(b => b.journalIds.indexOf(id) != -1)
            return
        }

        if (state.bankPaymentTypes.length > 0) {
            state.selectedBankPaymentType = this.getters.bankPaymentTypes[0]
            return
        }

        state.selectedBankPaymentType = null
    }
}

const actions = {
    initBankPaymentTypes ({ commit }, payload) {
        commit('setBankPaymentTypes', payload)
        commit('setSelectedBankPaymentType')
        commit('setLoadingBankPaymentTypes')
    },
    changeBankPaymentType({ commit }, id) {
        commit('setSelectedBankPaymentType', id)
    },
    resetBankPaymentType ({ getters, commit }) {
        if (getters.isWired) {
            commit('setLoadingBankPaymentTypes', true)
            commit('setBankPaymentTypes', [])
        }
        
        commit('setSelectedBankPaymentType')
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}