const state = {
    saleOrders: [],
    filteredSaleOrders: [],
    loadingSaleOrders: false,
    selectedSaleOrder: null
}

const getters = {
    loadingSaleOrders(state) {
        return state.loadingSaleOrders
    },
    saleOrders(state) {
        return state.saleOrders
    },
    visibleSaleOrders(state) {
        return state.filteredSaleOrders.length === 0 ? state.saleOrders : state.filteredSaleOrders
    },
    selectedSaleOrder(state) {
        return state.selectedSaleOrder
    }
}

const mutations = {
    setLoadingSaleOrders(state, loading) {
        state.loadingSaleOrders = !!loading
    },
    setSaleOrders(state, saleOrders) {
        state.saleOrders = saleOrders
    },
    setFilteredSaleOrders(state, payload) {
        state.filteredSaleOrders = [...payload]
    },
    setSelectedSaleOrder(state, saleOrder) {
        state.selectedSaleOrder = saleOrder
    }
}

const actions = {
    initSaleOrders({ commit }, saleOrders) {
        commit('setSaleOrders', saleOrders)
        commit('setLoadingSaleOrders')
    },
    filterSaleOrders({ commit }, payload) {
        commit('setFilteredSaleOrders', payload)
    },
    selectSaleOrder({ commit, dispatch }, payload) {
        commit('setSelectedSaleOrder', payload)
        dispatch('updateFromSaleOrder', payload)
    },
    resetSaleOrder({ commit }) {
        commit('setLoadingSaleOrders', true)
        commit('setSaleOrders', [])
        commit('setSelectedSaleOrder', null)
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}